/*exec.h******************************************************************
**version 0.2
*/


#ifndef _EXEC_H_
#define _EXEC_H_

/* Platform specific configuration for this section:
** - I think the definition for INLINE belongs in some common
**   header for all modules in the game, do we have one yet?  
** - EINT32 is an exactly 32 bit integer for use in ENodes.
*/

#ifdef SOMEHOW_DETECT_GCC

  /* GCC */
  #include "This section for GCC not yet checked"
  #define INLINE inline
  typedef EINT32 XXXXXXX;

#else
#ifdef _MSC_VER

  /* Microsoft C (Used to test the alife section, but will not be
  ** supported in the final game - Toby)
  */
  #define INLINE
  typedef long EINT32;

#else

  #include "No definitions for this platform are provided yet"

#endif  
#endif



/*
** Def'ns for execution trees
*/

#define STACK_SIZE 1000

union Stack_Value;
struct ENode;
struct Tab;

typedef int ExeFun(struct ENode *where);
typedef void SerializeFun(struct ENode *where,FILE *file,struct Tab *tab);
typedef struct ENode *DeSerializeFun(struct ENode *where,FILE *file);
typedef void DeleteFun(struct ENode *where);
typedef int VolumeFun(struct ENode *where,int paint);

union Stack_Value {
  EINT32 i;
  float f;  /* I didnt know we were using floats at all? - Toby */
  void *p;
};

/* Limit on length of the ENode name strings */
#define ENODE_NAME_LEN 20

/* This table is equivalent to a C++ virtual methods table;
** one of these tables is created for each type of ENode. Instances
** of that enode type contain a pointer to the table.
** In addition to pointers to member functions, this table also
** contains a pointer to the ENodes name        - Toby
*/
struct ENodeFunctions {
  ExeFun *exe;                  /* Call this function to evaluate node */
  SerializeFun *serialize;      /*   ...to save to disk */
  DeSerializeFun *deserialize;  /*   ...to restore from disk */
  VolumeFun *volume;            /*   ...to count nodes under this one */
  DeleteFun *delete;            /*   ...to delete this and its children */

  const char name[ENODE_NAME_LEN+1]; /* Name of node */
  
  const struct ENodeFunctions *next; /* These ENodeFunctions blocks are */
                                     /* stored in a linked list too */
};

/* Linked ENodes form the GP tree. Links to children are stored in
** a malloced structure kept in void *private. These stuctures should
** be allocated using the enode_malloc and enode_free functions in
** exec.c
*/
struct ENode {
  void *private;
  const struct ENodeFunctions *functions;
};

/* struct Tabs are created as local variables during serialization
** to determine the tabstop information when drawing the enode tree.
*/
struct Tab {
  struct Tab *prev;
  int enabled;
  int state;
  int gap;
};

#ifdef _EXEC_C_

union Stack_Value Stack[STACK_SIZE];
union Stack_Value *StackPointer = Stack;
union Stack_Value *FramePointer = Stack;

#else

extern union Stack_Value Stack[];
extern union Stack_Value *StackPointer;
extern union Stack_Value *FramePointer;

#endif
#endif




